<?php namespace Myriad\Installations\Validation;

use Illuminate\Contracts\Validation\Factory;
use Illuminate\Support\Arr;
use Myriad\Installations\Exceptions\InstallationValidationException;
use Ramsey\Uuid\Uuid;

class InstallationValidationProvider implements IInstallationValidationProvider {

  /**
   * @var Factory
   */
  private $validator;

  /**
   * InstallationValidationProvider constructor.
   *
   * @param Factory $validator
   */
  public function __construct(Factory $validator) {
    $this->validator = $validator;
  }

  /**
   * @inheritdoc
   */
  public function getRules($attributes = []) {
    return [
      'uuid' => 'required|uuid',
      'app_name' => 'required|max:64',
      'app_version' => 'required|max:64',
      'platform_name' => 'in:android,ios,web',
      'platform_version' => 'max:32',
      'locale' => 'locale',
      'timezone' => 'timezone',
      'push_type' => 'in:parse,gcm,apns',
      'push_id' => 'max:255',
      'user_agent' => 'required'
    ];
  }

  /**
   * @inheritdoc
   */
  public function getMessages($attributes = []) {
    return [];
  }

  /**
   * @inheritdoc
   */
  public function getValidator($attributes) {
    return $this->validator->make($attributes, $this->getRules($attributes), $this->getMessages($attributes));
  }

  /**
   * Validate a uuid
   *
   * @param $uuid
   * @return true
   * @throws InstallationValidationException
   */
  public function validateUUID($uuid) {
    $uuidRules = Arr::get($this->getRules(['uuid' => $uuid]), 'uuid');
    $uuidMessages = Arr::get($this->getMessages(['uuid' => $uuid]), 'uuid');
    if (!empty($uuidRules)) {
      $validator = $this->validator->make(['uuid' => $uuid], ['uuid' => $uuidRules], empty($uuidMessages) ? [] : $uuidMessages);
      if ($validator->fails()) {
        throw new InstallationValidationException($validator);
      }
    }
    return true;
  }

  /**
   * @inheritdoc
   */
  public function normalizeUUID($uuid) {
    try {
      return strtolower(Uuid::fromString($uuid)->toString());
    } catch (\Exception $e) {
      return $uuid;
    }
  }

}
